<?php
// $Id: XoxoFactory.php,v 1.1.4.1 2010/11/28 22:33:19 pounard Exp $

class XoxoFactoryException extends OoxException { }

/**
 * Default factory for XoxoObject instances.
 * 
 * The factory is able to create, read, update or insert any objects of the
 * factory target type, using transactionnal requests for create and update
 * operations.
 * Target type is a class name, given by the hook_xoxo_factory() module
 * custom implementation that defines it.
 * 
 * Factories are themselves registrered using the XoxoFactoryRegistry
 * specific OoxRegistry implementation. This is masked by some helpers
 * in the XoXo module such as xoxo_factory_get().
 * 
 * @see oox_factory_get()
 * @see XoxoFactoryRegistry
 * @see OoxRegistry
 */
class XoxoFactory
{
  // An error happend during save.
  const SAVED_ERROR = 0;
  // Object has been updated.
  const SAVED_UPDATED = 1;
  // Object has been saved.
  const SAVED_NEW = 2;

  // Clone object when loading it from cache.
  const LOAD_METHOD_CLONE = 1;
  // Get instance when loading it from cache.
  const LOAD_METHOD_INSTANCE = 2;

  // FIXME: Implement this.
  // Do not cache objects.
  const CACHE_METHOD_NONE = 0;
  // Cache object on a per-object basis.
  const CACHE_METHOD_OBJECT = 1;
  // Cache all known objects in one single cache entry.
  const CACHE_METHOD_ALL = 2;

  /**
   * Internal type name, usefull for the registry pattern.
   * 
   * @var string
   */
  private $__type = NULL;

  /**
   * Get internal type name.
   * 
   * @return string
   */
  public function getType() {
    return $this->__type;
  }

  /**
   * Item registry type.
   * 
   * @var string
   */
  private $__itemRegistryType;

  /**
   * Supported registry item types.
   * 
   * @var array
   *   Keys are be types, values are the human readable names.
   */
  private $__supportedItemTypes;

  /**
   * Returns the full range of supported registry item types.
   * 
   * @return array
   *   Keys are be types, values are the human readable names.
   */
  public final function getSupportedItemTypes() {
    if (isset($this->__supportedItemTypes)) {
      return $this->__supportedItemTypes;
    }
    else {
      return $this->getItemRegistry()->getItemTypes();
    }
  }

  /**
   * Tell if given item type is supported. If you have only one check to do,
   * call this method, it will be faster than getting the full range then
   * check in it.
   * 
   * @return boolean
   */
  public final function isItemTypeSupported($type) {
    return !isset($this->__supportedItemTypes) || isset($this->__supportedItemTypes[$type]);
  }

  /**
   * Get the registry that manage this factory's objects instances.
   * 
   * @return OoxRegistry
   */
  public final function getItemRegistry() {
    return OoxRegistry::getInstance($this->__itemRegistryType);
  }

  /**
   * SQL table name.
   * 
   * @var string
   */
  private $__tableName;

  /**
   * Get table name.
   * 
   * Override this method if you don't define the table name at hook level.
   * 
   * @return string
   *   Table name.
   */
  public function getTableName() {
    if (!isset($this->__tableName)) {
      throw new XoxoFactoryException("No table name set.");
    }
    return $this->__tableName;
  }

  /**
   * Tells if target class is exportable.
   * 
   * FIXME: really fix me, this is important.
   */
  public final function isExportable() {
    // return is_subclass_of($this->_className, 'XoxoExportableObject');
    return FALSE;
  }

  /**
   * Cache method.
   * 
   * @var int
   */
  protected $_cacheMethod = self::CACHE_METHOD_NONE;

  /**
   * Load method.
   * 
   * @var int
   */
  protected $_loadMethod = self::LOAD_METHOD_INSTANCE;

  /**
   * Object type machine name.
   * 
   * @var string
   */
  protected $_type;

  /**
   * Construct the factory using the given table name.
   *
   * @param array $item
   *   Descriptive registry type array.
   */
  public function __construct(&$item) {
    $this->__tableName = $item['table'];
    $this->_type = $item['type'];
    $this->__itemRegistryType = isset($item['registry_type']) ? $item['registry_type'] : $item['type'];
    if ($item['load_method']) {
      $this->_loadMethod = $item['load_method'];
    }
    $this->__supportedItemTypes = $item['item_type'];
  }

  /**
   * Maintain a loaded object cache.
   * 
   * @var array
   */
  protected $_loaded = array();

  /**
   * Populate new instance using given database row.
   * This is an internal method, please avoid relying on it.
   * 
   * Any static cache fill at load time must be implemented in this method for
   * overriding caches, this is the only entry point for all object loading.
   * 
   * @param object $row
   *   Full row from database.
   * 
   * @return XoxoExportableObject
   *   New object from given row data.
   */
  protected function _populate($row) {
    $object = unserialize($row->data);
    if ($object instanceof XoxoObject) {
      // Override some parameters that could have been saved with database
      // specified data. This will allow modules to alter data in database
      // without having to load/save the full object and ensure the real
      // loaded instance will keep these settings.
      $object->setIdentifier($row->oid);
      $object->setOwner($row->uid);
      $object->setStatus($row->status);
      $object->setDefault($row->defaults);
      // Load object options, if any.
      if ($row->options_oid) {
        OptionsFactory::getInstance()->load($row->options_oid, $object);
      }
      // Fill the object cache.
      $this->_cacheAdd($object);
      return $object;
    }
    return NULL;
  }

  /**
   * An object has been loaded, overriding classes should implement this in
   * order to update their internal static cache.
   * 
   * It could be done in _populate() method, but in some rare cases, objects
   * can be loaded without going through _populate().
   */
  protected function _objectLoaded(XoxoObject $object) {
    $object->restore();
  }

  /**
   * Load an object by identifier.
   * 
   * @param int $oid
   *   Identifier.
   * 
   * @return XoxoObject
   *   XoxoObject instance.
   * 
   * @throws XoxoFactoryException
   *   If layout does not exists.
   */
  public final function loadByIdentifier($oid) {
    $object = NULL;
    if (!isset($this->_loaded[$oid])) {
      // Load from database aptempt.
      $data = db_select($this->getTableName(), 'ct')
        ->fields('ct')
        ->condition('oid', $oid)
        ->execute()
        ->fetchObject();
      if ($data) {
        // This will trigger object caching.
        $this->_objectLoaded($this->_populate($data));
      }
      else {
        throw new XoxoFactoryException("Object " . $oid . " does not exists.");
      }
    }
    $ret = NULL;
    switch ($this->_loadMethod) {
      case self::LOAD_METHOD_CLONE:
        $ret = clone($this->_loaded[$oid]);
        break;
      case self::LOAD_METHOD_INSTANCE:
      default:
        $ret = $this->_loaded[$oid];
    }
    return $ret;
  }

  /**
   * Load all objects owner by this particular user.
   * 
   * @see XoxoFactory::loadAll()
   *   For addition parameters.
   * 
   * @param int|object $account
   *   Account identifier or object.
   * @params ...
   *   XoxoFactory::loadAll() parameters.
   */
  public final function loadAllByOwner($account, $pagerLimit = 0, $excludeDisabled = TRUE, $pagerElement = 0) {
    $uid = is_object($account) ? $account->uid : $account;
    return $this->_loadAll($pagerLimit, $excludeDisabled, $pagerElement, array('uid' => $uid));
  }

  /**
   * Load all objects.
   * 
   * Warning: do not use this method if you can do otherwise. This could be
   * resource intensive if you do not use paging.
   * 
   * @param int $pagerLimit = 0
   *   (optional) Set here the limit for paged results. This will make the
   *   factory use the Drupal pager_query() in order to get back result, this
   *   particular case implies that the page parameter is given as GET param,
   *   so you won't have control over it.
   *   Notice: by using the pagerLimit, you deactivate the object exclusion
   *   mecanism from query to ensure you have a full range of objects.
   * @param boolean $excludeDisabled = TRUE
   *   (optional) If set to FALSE, disabled object will be load too. This is
   *   useful for administrator forms or administration pages.
   * @param int $pagerElement = 0
   *   (optional) For administration pages control, you can set here the pager
   *   element value.
   * 
   * @return array
   *   Array of XoxoObject. You can't rely on keys since the overriding
   *   classes may arbitrary add elements.   
   */
  public function loadAll($pagerLimit = 0, $excludeDisabled = TRUE, $pagerElement = 0) {
    return $this->_loadAll($pagerLimit, $excludeDisabled, $pagerElement);
  }

  /**
   * Method that should be overiden, this allows to exclude cached content
   * from loadAll() requests to avoid objects double loading.
   * 
   * This is run in loadAll() internal method, before additional filters
   * building.
   * 
   * This function will only be called if query is not paged. Dropping results
   * from the paged query would arbitrary change the result number for the
   * current page.
   * 
   * See this sample implementation to see how to implement it. Don't forget
   * to call parent::_loadAllExcludeFromCache() and merge your results when
   * you override it.
   * 
   * @return array
   *   Key/value pairs, keys are field names, values are an array of values
   *   to exclude for this field.
   */
  protected function _loadAllExcludeFromCache() {
    return array('oid' => array_keys($this->_loaded));
  }

  /**
   * Tells if a field exists in schema API for the current object type.
   * 
   * @return boolean
   */
  protected final function _fieldExists($field) {
    $schema = drupal_get_schema($this->getTableName());
    return !empty($schema) && isset($schema['fields'][$field]);
  }

  /**
   * This will create the query conditions that matches the given access
   * permission you can then use as the $condition parameter for load
   * methods.
   * 
   * @param string $op
   *   'delete', 'update' or 'view'. Object creation here as no sens because
   *   only existing objects are going to be loaded from database.
   * @param int|object $account = NULL
   *   (optional) Account identifier or object. If NULL it will uses the
   *   current user for access checks.
   * 
   * @return array
   *   Key/value pairs of parameters. Keys are field name, and values are
   *   values to match in the query.
   *   Returns FALSE in case an invalid permission is given, or if the user
   *   has no right at all.
   */
  protected final function _getUserAccessConditions($op, $account = NULL) {
    // Check if an account was given. We need the full object structure
    // for the user_access method.
    if (!$account) {
      global $user;
      $account = $user;
    }
    else {
      $account = is_object($account) ? $account : user_load($account);
    }
    // Do the proper object exclusions following the permission.
    $perm_any = $op . ' any ' . $this->_type;
    $perm_own = $op . ' own ' . $this->_type;
    // Check user access.
    if (user_access($perm_any)) {
      // No filtering when 'any' type permission is given.
      return array();
    }
    else if (user_access($perm_own)) {
      return array('uid' => $account->uid);
    }
    else {
      return FALSE;
    }
  }

  /**
   * Load all internal implementation. This will allow other modules to use it
   * and alter query in order to add different or additional query filters.
   * 
   * @see XoxoFactory::loadAll()
   *   For parameters extended explanation.
   * 
   * @param int $pagerLimit = 0
   * @param boolean $excludeDisabled = TRUE
   * @param int $pagerElement = 0
   * @param array $conditions = array()
   *   (optional) Key/value pairs of parameters. Keys are field names, and
   *   values are field value to match in query.
   *   Set of additional parameters to append to the query. Notice this is not
   *   the same as db_* functions.
   *   You don't have to specify database placeholder, the method introspects
   *   schema API to find out which one to use. This is also a security check
   *   that will avoid to query non existing fields.
   *   If you are willing to create complex queries with join statements or
   *   anything else a bit more tricky, you should write your own full loadAll
   *   method.
   *   You can set here an arbitrary 'user_access', this will force this method
   *   to check current user access for objects. Allowed values are 'view',
   *   'update' or 'delete'.
   */
  protected final function _loadAll($pagerLimit = 0, $excludeDisabled = TRUE, $pagerElement = 0, $conditions = array()) {
    // Prepare the query.
    $alias = 'ct';
    $select = db_select($this->getTableName(), $alias)->fields('ct');

    // Get a cache of what result returns, in for paging iteration.
    $all = FALSE;
    $fetched = array();

    // First aptempt by excluding already loaded objects.
    // We are not going to exclude from cache as soon as we have some filters
    // because we would not be able to restore cached elements that matches.
    if (!$pagerLimit && !empty($this->_loaded) && empty($conditions)) {
      $all = TRUE;
      // Exclude already loaded objects from query, this could help when working
      // with a lot of objets in the same PHP execution flow.
      foreach ($this->_loadAllExcludeFromCache() as $field => $values) {
        // Exclude empty exclusions arrays from query, and also check given
        // field exists in Drupal schema API.
        if (!empty($values) && $this->_fieldExists($field)) {
          $select->condition($field, $values, 'NOT IN');
        }
      }
    }

    // Check for status exclusion.
    if ($excludeDisabled) {
      $select->condition('status', XoxoObject::STATUS_DISABLED, '<>');
    }

    // Check for user access.
    if (isset($conditions['user_access'])) {
      // Fetch new fields associated conditions.  
      $access_conditions = $this->_getUserAccessConditions($value);
      if (!is_array($access_conditions)) {
        // If user has no rights at all, just return an empty array (nothing
        // to load here, it will, at least, avoid one SQL request).
        return array();
      }
      else {
        foreach ($access_conditions as $field => $values) {
          $select->condition($field, $values, (is_array($values) ? 'IN' : '='));
        }
      }
    }

    // Append other condition parameters given.
    foreach ($conditions as $field => $values) {
      $select->condition($field, $values, (is_array($values) ? 'IN' : '='));
    }

    // Do the real query.
    if ($pagerLimit) {
      // Ok, this is ugly, but totally legal.
      $select = $select
        ->orderBy('oid', 'ASC')
        ->extend('PagerDefault')
        ->limit($pagerLimit)
        ->element($pagerElement);
    }

    // Where the magic happens.
    $result = $select->execute();

    // Iterate over loaded object in order to load them all.
    while ($data = $result->fetchObject()) {
      $object = $this->_populate($data);
      $this->_objectLoaded($object);
      // Append new result to fetched array. This will ensure we return only
      // the fetched elements.
      $fetched[$object->getIdentifier()] = $object;
    }

    // If no pager limit is given, the users wants the full database. This
    // seems quite dumb, but give it to him won't kill kill us, it will only
    // kill its site performances.
    if ($all) {
      $fetched = &$this->_loaded;
    }

    $objects = array();

    // New iteration over cache, to ensure we got all objects including those
    // we already loaded before.
    foreach ($fetched as &$object) {
      // Even if the query has already excluded disabled objects, they may have
      // been arbitrary loaded during the execution flow.
      if ($all && $excludeDisabled && $object->getStatus() == XoxoObject::STATUS_DISABLED) {
        continue;
      }

      // Proceed to load method fetching.
      switch ($this->_loadMethod) {
        case self::LOAD_METHOD_CLONE:
          $objects[] = clone($object);
          break;
        case self::LOAD_METHOD_INSTANCE:
        default:
          // We can afford a reference here, because when returning the array
          // it will be copied.
          $objects[] = $object;
      }
    }
    return $objects;
  }

  /**
   * Save a layout instance into database. This will not save included
   * content.
   * 
   * @param XoxoExportableObject $object
   * 
   * @throws XoxoFactoryException
   *   In case of invalid instance given.
   */
  public final function save(XoxoObject $object) {
    $insert = FALSE;

    // Get a clean object clone, and set data for database saving.
    $prepared = clone($object);
    $prepared->clean();
    $data = array(
      'data' => serialize($prepared),
      'uid' => $object->getOwnerUid(),
      'status' => $object->getStatus(),
    );

    // Try to get back current identifier, if none exists, then we are doing an
    // insert.
    try {
      $oid = $object->getIdentifier();
      $update = array('oid');
      $data['oid'] = $oid;
      $data['updated'] = time();
      // An updated object is always a non default.
      $data['defaults'] = 0;
      $ret = self::SAVED_UPDATED;
    }
    catch (OoxException $e) {
      $insert = TRUE;
      $update = array();
      $data['updated'] = $data['created'] = time();
      $data['defaults'] = $object->isDefault() ? 1 : 0;
      $ret = self::SAVED_NEW;
    }

    // Create a transaction (this won't work with MyISAM) to ensure we can set
    // the name property to the oid property.
    // We do this because when operating on XoxoObject, name has no sense
    // name must be used only for exportables that needs a unique machine name.
    // Note that in any case, there really a few chances that this transaction
    // overlaps with another, so, if you are lucky, it may still work using the
    // MyISAM MySQL engine.
    $transaction = db_transaction();

    // Last chance modification for overriding classes.
    $this->_preSave($object, $data);

    try {
      // Real save.
      if (drupal_write_record($this->getTableName(), $data, $update) === FALSE) {
        $ret = self::SAVED_ERROR;
      }
      else {
        // This should always goes here.
        if (empty($data['name'])) {
          db_update($this->getTableName())
            ->fields(array('name' => $data['oid']))
            ->condition('oid', $data['oid'])
            ->execute();
        }
        $this->_postSave($object, $data);
      }

      // If no error, and we are in the case of object creation, update cache.
      // In case of object update, we don't need it because object reference is
      // already in the static cache. Clones are BAD, and should never be done
      // by other modules.
      if ($insert) {
        $object->setIdentifier($data['oid']);
        $this->_objectLoaded($object);
      }
      else {
        // Reset default flag at update.
        $object->setDefault(FALSE);
      }

      // Update the cache if needed.
      $this->_cacheAdd($object);
    }
    catch (OoxException $e) {
      watchdog_exception('xoxo_factory', $e);
      $ret = self::SAVED_ERROR;
      $transaction->rollback();
    }
    catch (PDOException $e) {
      watchdog_exception('xoxo_factory', $e);
      $ret = self::SAVED_ERROR;
      $transaction->rollback();
    }

    return $ret;
  }

  /**
   * Last chance for overriding classes to alter the $data array when saving.
   * This is an internal method, please do not override it unless you really
   * know what you are doing.
   * 
   * This function is run within a database transaction, any request that
   * does an error here will then implies a database rollback and object
   * save failure.
   * 
   * @param XoxoObject $object
   *   Object being saved. If this is an insert, the object does not have
   *   its identifier set yet.
   * @param array &$data
   *   The data array that will be passed to drupal_write_record().
   */
  protected function _preSave(XoxoObject $object, &$data) { }

  /**
   * Save has been done, with no errors.
   * 
   * This function is run within a database transaction, any request that
   * does an error here will then implies a database rollback and object
   * save failure.
   * 
   * @param XoxoObject $object
   *   Object being saved. If this is an insert, the object does not have
   *   its identifier set yet.
   * @param array &$data
   *   The data array that got through to drupal_write_record(), identifier
   *   is set in this array as $data['oid'].
   */
  protected function _postSave(XoxoObject $object, &$data) {
    // Save all options.
    OptionsFactory::getInstance()->save($object);
  }

  /**
   * Delete object.
   * 
   * Using the identifier to delete an object is mandatory, because other
   * factories may want to delete other stuff arround. Delete has no error
   * handling, once the object has been deleted, there is no comming back.
   * 
   * @see XoxoFactory::_postDelete()
   * 
   * @param int $oid
   *   XoxoObject identifier.
   * 
   * @return int
   *   Number of rows deleted.
   */
  public final function deleteByIdentifier($oid) {
    return $this->deleteByField(array('oid' => $oid));
  }

  /**
   * Every factory that uses secondary database tables to store data arround
   * the objects should use this method to clean their data. Use this method
   * to also cleanup your caches.
   * 
   * Their is no transaction here and object has already been deleted, just
   * delete all remaining garbage.
   * 
   * @see XoxoFactory::deleteByIdentifier()
   * 
   * @param int $oid
   *   XoxoObject identifier.
   */
  protected function _postDelete($oid) { }

  /**
   * Delete objects using the given table field. This field must exist in
   * current table schema API cache.
   * 
   * It will do two requests, the first one will fetch back identifiers so
   * we can erase only a part of the cache, the second one will really delete
   * those objects using a IN statement for identifiers, to ensure no database
   * update have been done between our two requests.
   * 
   * @param array $conditions
   *   Array of conditions, each value is a field name, each value is a field
   *   value.
   * 
   * @return int
   *   Number of rows deleted.
   */
  public function deleteByField($conditions) {
    $deleted = 0;
    $table = $this->getTableName();

    $transaction = db_transaction();

    try {
      // First request.
      $select = db_select($table, 'ct')
        ->fields('ct', array('oid'));
      foreach ($conditions as $field => $value) {
        $select->condition($field, $value);
      }
      $oids = $select->execute()->fetchCol();
  
      // Proceed only if we have something to delete.
      if ($deleted = count($oids)) {
        // Second query.
        db_delete($table)
          ->condition('oid', $oids, 'IN')
          ->execute();

        // Force transaction to commit.
        unset($transaction);

        // Then also clear nicely the cache.
        $this->cacheClear($oids);
      }
    }
    catch (PDOException $e) {
      // This should not happen.
      watchdog_exception('xoxo_factory', $e);
    }

    return $deleted;
  }

  /**
   * Delete objects owned by the given account.
   * 
   * @param int|object $account
   *   Account identifier or object.
   * 
   * @return int
   *   Number of rows deleted.
   */
  public function deleteByOwner($account) {
    $uid = is_object($account) ? $account ->uid : $account;
    return $this->deleteByField(array('uid', $uid));
  }

  /**
   * Check if the given identifier exists in database.
   * 
   * @param int $oid
   *   Identifier.
   * 
   * @return boolean
   */
  public function existsByIdentifier($oid) {
    return $this->_cacheHas($oid) || (bool) db_result(db_query("SELECT 1 FROM {:table} WHERE oid = :oid", array(':table' => $this->getTableName(), ':oid' => $oid)));
  }

  /**
   * Factory method used by common code. This should be overriden by any
   * specific factory implementation.
   * 
   * @param string $type
   *   Registry item type. If given, will spawn a factory named using the
   *   factory type as registry type and aptempt to create the given item
   *   type as class to instanciate the object.
   *
   * @return XoxoObject
   *   New empty instance.
   * 
   * @throws OoxRegistryException
   *   When item is not supported by the registry itself.
   * @throws XoxoFactoryException
   *   When type is not supported.
   */
  public final function createInstance($type) {
    if ($this->isItemTypeSupported($type)) {
      return $this->getItemRegistry()->getItem($type);
    }
    throw new XoxoFactoryException("Unsupported item type " . $type);
  }

  /**
   * Change only object status in database, leaving all other stuff unchanged.
   * Status must have already been changed in object.
   * 
   * @param XoxoObject $object
   */
  public function saveStatus(XoxoObject $object) {
    db_update($this->getTableName())
      ->fields(array('status' => $object->getStatus()))
      ->condition('oid', $object->getIdentifier())
      ->execute();
  }

  /**
   * Tell if the current object is in cache.
   * 
   * Overriding classes that carries different caching mecanism should override
   * this method. If they don't, this won't affect the code, except some
   * operations will require more database queries.
   * 
   * If ensure, leave default, it's always better to return FALSE here instead
   * of TRUE, wrong FALSE statements will trigger queries, wrong TRUE statements
   * will cause a wrong fonction return.
   * 
   * @param int $oid
   *   Object identifier to test.
   * 
   * @return boolean
   */
  protected function _cacheHas($oid) {
    return isset($this->_loaded[$oid]);
  }

  /**
   * Add a loaded object to cache.
   * 
   * For overriding classes, this is the right method to override to add your
   * own cache mecanism and implementation.
   * 
   * If you override it, please call the parent method or update carefully
   * the protected _loaded property.
   * 
   * @param XoxoObject $object
   *   Object to add to static cache.
   */
  public function _cacheAdd(XoxoObject $object) {
    $this->_loaded[$object->getIdentifier()] = $object;
  }

  /**
   * Clear internal static cache.
   * 
   * @param array $oids = array()
   *   (optional) Objects to remove from static cache.
   */
  public final function cacheClear($oids = array()) {
    $this->_cacheClear($oids);
  }

  /**
   * Clear internal static cache.
   * 
   * The first parameter will allow the method to clear only given objects from
   * static cache. Any overriding class may override this method to ensure that
   * all caches are cleared. When implementing it, if you sense that clearing
   * correctly your caches could take more time than reloading the objects, then
   * erase it all. Remember it's better to clean it completely than let a wrong
   * static cache living.
   * 
   * If you override it, please call the parent method or update carefully
   * the protected _loaded property.
   * 
   * @param array $oids = array()
   *   (optional) Objects to remove from static cache.
   */
  protected function _cacheClear($oids = array()) {
    if (!empty($oids)) {
      foreach ($oids as $oid) {
        // See below notice for living instances.
        unset($this->_loaded[$oid]);
      }
    }
    else {
      // This will keep out of scope every objects in the cache, so that the
      // __destruct() magic method will be called. It wont be only in case
      // some of them are still be used by other modules.
      $this->_loaded = array();
    }
  }

  /**
   * Get help text that will display on administration overview page.
   * 
   * FIXME: Quite ugly, find a better pattern than this.
   * 
   * @return string
   *   Human readable localized string.
   */
  public function getOverviewHelp() { return ""; }
}
